@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.1
+ *  CU sudo version 1.5.2
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: tgetpass.c,v 1.47 1996/09/08 00:21:42 millert Exp $";
+static char rcsid[] = "$Id: tgetpass.c,v 1.49 1996/10/08 22:54:49 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -48,9 +48,9 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.47 1996/09/08 00:21:42 millert Exp $"
 #ifdef HAVE_SYS_BSDTYPES_H
 #include <sys/bsdtypes.h>
 #endif /* HAVE_SYS_BSDTYPES_H */
-#ifdef _AIX
+#ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
-#endif /* _AIX */
+#endif /* HAVE_SYS_SELECT_H */
 #include <sys/time.h>
 #include <signal.h>
 #include <fcntl.h>
